#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file Robot1.py
 @brief FSM4RTC Test
 @date $Date$

 @author 宮本　信彦　n-miyamoto@aist.go.jp
 産業技術総合研究所　ロボットイノベーション研究センター
 ロボットソフトウエアプラットフォーム研究チーム

 LGPL

"""
import sys
import time
sys.path.append(".")

# Import RTM module
import RTC
import OpenRTM_aist

import OpenRTM_aist.StaticFSM as StaticFSM
import OpenRTM_aist.EventPort as EventPort
from OpenRTM_aist.Macho import *


from Robot1Fsm import *
from robot_hard import *


# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
robot1_spec = ["implementation_id", "Robot1", 
		 "type_name",         "Robot1", 
		 "description",       "FSM4RTC Test", 
		 "version",           "1.0.0", 
		 "vendor",            "AIST", 
		 "category",          "Test", 
		 "activity_type",     "STATIC", 
		 "max_instance",      "1", 
		 "language",          "Python", 
		 "lang_type",         "SCRIPT",
		 "conf.default.start_pos", "-0.3",
		 "conf.default.size", "0.1",
		 "conf.default.speed", "0.1",
		 "conf.default.K", "0.1",
		 "conf.default.target", "0.6",
		 "conf.default.EPS", "0.01",
		 "conf.default.box_size", "0.1",
		 "conf.default.box_startpos", "0",

		 "conf.__widget__.start_pos", "text",
		 "conf.__widget__.size", "text",
		 "conf.__widget__.speed", "text",
		 "conf.__widget__.K", "text",
		 "conf.__widget__.target", "text",
		 "conf.__widget__.EPS", "text",
		 "conf.__widget__.box_size", "text",
		 "conf.__widget__.box_startpos", "text",

         "conf.__type__.start_pos", "double",
         "conf.__type__.size", "double",
         "conf.__type__.speed", "double",
         "conf.__type__.K", "double",
         "conf.__type__.target", "double",
         "conf.__type__.EPS", "double",
         "conf.__type__.box_size", "double",
         "conf.__type__.box_startpos", "double",

		 ""]
# </rtc-template>

##
# @class Robot1
# @brief FSM4RTC Test
# 
# FSM4RTCのサンプル
# 
# 
class Robot1(OpenRTM_aist.DataFlowComponentBase):
	
	##
	# @brief constructor
	# @param manager Maneger Object
	# 
	def __init__(self, manager):
		OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

		
		
		
		
		self._d_wait1 = RTC.TimedDouble(RTC.Time(0,0),0.0)
		"""
		待ち状態通知イベント送信
		 - Type: RTC::TimedLong
		"""
		self._wait1Out = OpenRTM_aist.OutPort("wait1", self._d_wait1)
		
		self._d_begin = RTC.TimedDouble(RTC.Time(0,0),0.0)
		"""
		搬送開始イベント送信
		 - Type: RTC::TimedLong
		"""
		self._beginOut = OpenRTM_aist.OutPort("begin", self._d_begin)
		
		self._d_move = RTC.TimedDouble(RTC.Time(0,0),0.0)
		"""
		搬送時の移動イベント送信
		 - Type: RTC::TimedLong
		"""
		self._moveOut = OpenRTM_aist.OutPort("move", self._d_move)
		
		self._d_lost1 = RTC.TimedDouble(RTC.Time(0,0),0.0)
		"""
		荷物ロスト通知イベント送信
		 - Type: RTC::TimedLong
		"""
		self._lost1Out = OpenRTM_aist.OutPort("lost1", self._d_lost1)


		


		# initialize of configuration-data.
		# <rtc-template block="init_conf_param">
		"""
		初期位置
		 - Name: start_pos start_pos
		 - DefaultValue: -0.3
		 - Unit: m
		"""
		self._start_pos = [-0.3]
		"""
		ロボットのサイズ
		 - Name: size size
		 - DefaultValue: 0.1
		 - Unit: m
		"""
		self._size = [0.1]
		"""
		最大速度
		 - Name: speed speed
		 - DefaultValue: 0.1
		 - Unit: m/s
		"""
		self._speed = [0.1]
		"""
		ゲイン定数
		 - Name: K K
		 - DefaultValue: 0.1
		"""
		self._K = [0.1]
		"""
		目標位置
		 - Name: target target
		 - DefaultValue: 0.6
		 - Unit: m
		"""
		self._target = [0.6]
		"""
		EPS
		 - Name: EPS EPS
		 - DefaultValue: 0.01
		 - Unit: m
		"""
		self._EPS = [0.01]
		"""
		荷物のサイズ
		 - Name: box_size box_size
		 - DefaultValue: 0.1
		 - Unit: m
		"""
		self._box_size = [0.1]
		"""
		荷物の初期位置
		 - Name: box_startpos box_startpos
		 - DefaultValue: 0.0
		 - Unit: m
		"""
		self._box_startpos = [0.0]
		
		# </rtc-template>


		 
	##
	#
	# The initialize action (on CREATED->ALIVE transition)
	# formaer rtc_init_entry() 
	# 
	# @return RTC::ReturnCode_t
	# 
	#
	def onInitialize(self):

		self.robot1 = Robot_Hard(self._start_pos[0], self._speed[0], self._K[0], self._EPS[0], self._size[0], 0.01, self._target[0])
		self.box = Box(self._box_startpos[0], self._box_size[0])


		"""
		イベント受信
		 - Type: RTC::TimedLong
		"""
		self._fsm = StaticFSM.Machine(Robot1_Top, self)
		self._fsm.dispatch(StaticFSM.Event(Robot1_Top.setRobot1Hard, self.robot1))
		self._fsm.dispatch(StaticFSM.Event(Robot1_Top.setBoxObj, self.box))
		self._fsm.dispatch(StaticFSM.Event(Robot1_Top.setWait1Port, self._d_wait1, self._wait1Out))
		self._fsm.dispatch(StaticFSM.Event(Robot1_Top.setBeginPort, self._d_begin, self._beginOut))
		self._fsm.dispatch(StaticFSM.Event(Robot1_Top.setMovePort, self._d_move, self._moveOut))
		self._fsm.dispatch(StaticFSM.Event(Robot1_Top.setLost1Port, self._d_lost1, self._lost1Out))
		
		
		self._eventIn = EventPort.EventInPort("event", self._fsm)
		self._eventIn.bindEvent1("sense1", Robot1_Top.sense, RTC.TimedDouble(RTC.Time(0,0),0.0),True)
		self._eventIn.bindEvent0("wait2", Robot1_Top.wait2,True)
		self._eventIn.bindEvent0("lost2", Robot1_Top.lost2,True)
		
		

				
		# Bind variables and configuration variable
		self.bindParameter("start_pos", self._start_pos, "-0.3")
		self.bindParameter("size", self._size, "0.1")
		self.bindParameter("speed", self._speed, "0.1")
		self.bindParameter("K", self._K, "0.1")
		self.bindParameter("target", self._target, "0.6")
		self.bindParameter("EPS", self._EPS, "0.01")
		self.bindParameter("box_size", self._box_size, "0.1")
		self.bindParameter("box_startpos", self._box_startpos, "0.0")
		
		# Set InPort buffers
		self.addInPort("event",self._eventIn)
		
		# Set OutPort buffers
		self.addOutPort("wait1",self._wait1Out)
		self.addOutPort("begin",self._beginOut)
		self.addOutPort("move",self._moveOut)
		self.addOutPort("lost1",self._lost1Out)
		
		# Set service provider to Ports
		
		# Set service consumers to Ports
		
		# Set CORBA Service Ports
		
		return RTC.RTC_OK



	def setRobot1(self, obj):
		self._fsm.dispatch(StaticFSM.Event(Robot1_Top.setRobot1Hard, obj))
		self.robot1 = obj
		

	def setBox(self, obj):
		self._fsm.dispatch(StaticFSM.Event(Robot1_Top.setBoxObj, obj))
		self.box = obj


	def getRobot1(self):
		return self.robot1
		

	def getBox(self):
		return self.box


	def getMachine(self):
		return self._fsm
		##
		# 
		# The finalize action (on ALIVE->END transition)
		# formaer rtc_exiting_entry()
		# 
		# @return RTC::ReturnCode_t
	
		# 
	def onFinalize(self):
		self._fsm.shutdown()
		return RTC.RTC_OK
	
		##
		#
		# The startup action when ExecutionContext startup
		# former rtc_starting_entry()
		# 
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onStartup(self, ec_id):
	
		return RTC.RTC_OK
	
		##
		#
		# The shutdown action when ExecutionContext stop
		# former rtc_stopping_entry()
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onShutdown(self, ec_id):
	
		return RTC.RTC_OK
	
		##
		#
		# The activated action (Active state entry action)
		# former rtc_active_entry()
		#
		# @param ec_id target ExecutionContext Id
		# 
		# @return RTC::ReturnCode_t
		#
		#
	def onActivated(self, ec_id):
		ec = self.getExecutionContext(ec_id)
		rate = 1.0/float(ec.get_rate())
		self.robot1.init(self._start_pos[0], self._speed[0], self._K[0], self._EPS[0], self._size[0], rate, self._target[0])
		self.box.init(self._box_startpos[0], self._box_size[0])
		
		self._fsm.dispatch(StaticFSM.Event(Robot1_Top.StateChange))
		return RTC.RTC_OK
	
		##
		#
		# The deactivated action (Active state exit action)
		# former rtc_active_exit()
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onDeactivated(self, ec_id):
	
		return RTC.RTC_OK
	
		##
		#
		# The execution action that is invoked periodically
		# former rtc_active_do()
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onExecute(self, ec_id):
		
		self._fsm.dispatch(StaticFSM.Event(Robot1_Top.tock))
		
		return RTC.RTC_OK
	
		##
		#
		# The aborting action when main logic error occurred.
		# former rtc_aborting_entry()
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onAborting(self, ec_id):
	
		return RTC.RTC_OK
	
		##
		#
		# The error action in ERROR state
		# former rtc_error_do()
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onError(self, ec_id):
	
		return RTC.RTC_OK
	
		##
		#
		# The reset action that is invoked resetting
		# This is same but different the former rtc_init_entry()
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onReset(self, ec_id):
	
		return RTC.RTC_OK
	
		##
		#
		# The state update action that is invoked after onExecute() action
		# no corresponding operation exists in OpenRTm-aist-0.2.0
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#

		#
	def onStateUpdate(self, ec_id):
	
		return RTC.RTC_OK
	
		##
		#
		# The action that is invoked when execution context's rate is changed
		# no corresponding operation exists in OpenRTm-aist-0.2.0
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onRateChanged(self, ec_id):
	
		return RTC.RTC_OK
	



def Robot1Init(manager):
    profile = OpenRTM_aist.Properties(defaults_str=robot1_spec)
    manager.registerFactory(profile,
                            Robot1,
                            OpenRTM_aist.Delete)

def MyModuleInit(manager):
    Robot1Init(manager)

    # Create a component
    comp = manager.createComponent("Robot1")

def main():
	mgr = OpenRTM_aist.Manager.init(sys.argv)
	mgr.setModuleInitProc(MyModuleInit)
	mgr.activateManager()
	mgr.runManager()

if __name__ == "__main__":
	main()

