#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

import sys

import RTC
import OpenRTM_aist
import OpenRTM_aist.StaticFSM as StaticFSM
import OpenRTM_aist.EventPort as EventPort
from OpenRTM_aist.Macho import *


import random


#robot1_loss_prob = 3
#robot1_lost_length = 0.5




@StaticFSM.fsm_topstate
class Robot1_Top(StaticFSM.Link):
  def on_entry(self):
    logger.debug("Robot1_Top::on_entry")
  def on_exit(self):
    logger.debug("Robot1_Top::on_exit")
  def on_init(self):
    logger.debug("Robot1_Top::on_init")
    #self.set_state(StaticFSM.State((Robot1_Approach)))
  def nextpos(self, d):
    pass
  def eqchk(self, x1, xb, eps):
    if(abs(x1-xb) < eps):
      return True
    else:
      return False

  def tock(self):
    pass
  def wait2(self):
    pass
  def begin(self):
    pass
  def lost2(self):
    pass

  def StateChange(self):
    logger.debug("Robot1_Top::StateChange")
    self.set_state(StaticFSM.State((Robot1_Approach)))
  def setRobot1Hard(self, robot1):
    self.data(Robot1_Top).robot1_hard = robot1
  def setBoxObj(self, box):
    self.data(Robot1_Top).box = box
  def sense(self, d):
    self.data(Robot1_Top).d = d.data
  def setWait1Port(self, data, port):
    self.data(Robot1_Top)._d_wait1 = data
    self.data(Robot1_Top)._wait1Out = port
  def setBeginPort(self, data, port):
    self.data(Robot1_Top)._d_begin = data
    self.data(Robot1_Top)._beginOut = port
  def setMovePort(self, data, port):
    self.data(Robot1_Top)._d_move = data
    self.data(Robot1_Top)._moveOut = port
  def setLost1Port(self, data, port):
    self.data(Robot1_Top)._d_lost1 = data
    self.data(Robot1_Top)._lost1Out = port
  
  class Data:
    def __init__(self):
      self.robot1_hard = None
      self.box = None
      self.d = 0
      self._d_wait1 = None
      self._wait1Out = None
      self._d_begin = None
      self._beginOut = None
      self._d_move = None
      self._moveOut = None
      self._d_lost1 = None
      self._lost1Out = None
      self.robot2_wait = False




@StaticFSM.fsm_substate(Robot1_Top)
class Robot1_Approach(StaticFSM.Link):
  def on_entry(self):
    logger.debug("Robot1_Approach::on_entry")
        
  def on_exit(self):
    logger.debug("Robot1_Approach::on_exit")
  def on_init(self):
    logger.debug("Robot1_Approach::on_init")

  def tock(self):
    logger.debug("Robot1_Approach::tock")
    self.nextpos(self.data(Robot1_Top).d)


  def sense(self, d):
    logger.debug("Robot1_Approach::sense")
    self.data(Robot1_Top).d = d.data
    robot1_hard = self.data(Robot1_Top).robot1_hard
    eps = robot1_hard._eps/2.0
    if self.eqchk(0,d.data,eps):
      self.set_state(StaticFSM.State(Robot1_Wait))
    
  def nextpos(self, d):
    logger.debug("Robot1_Approach::nextpos")
    robot1_hard = self.data(Robot1_Top).robot1_hard
    box = self.data(Robot1_Top).box
    #d = box.x - robot1_hard.x
    eps = robot1_hard._eps

    v = robot1_hard._speed*robot1_hard._sampling_time
      
    t = robot1_hard._K*d
    guard = OpenRTM_aist.ScopedLock(robot1_hard.mutex)
    if abs(t) > abs(v):
        robot1_hard.vel = v
    else:
      robot1_hard.vel = t



  def wait2(self):
    logger.debug("Robot1_Approach::wait2")
    self.data(Robot1_Top).robot2_wait = True
  def begin(self):
    pass





@StaticFSM.fsm_substate(Robot1_Top)
class Robot1_Wait(StaticFSM.Link):
  def on_entry(self):
    logger.debug("Robot1_Wait::on_entry")
        
  def on_exit(self):
    logger.debug("Robot1_Wait::on_exit")
  def on_init(self):
    logger.debug("Robot1_Wait::on_init")
    robot1_hard = self.data(Robot1_Top).robot1_hard
    robot1_hard.vel = 0
    port = self.data(Robot1_Top)._wait1Out
    port.write()
    if self.data(Robot1_Top).robot2_wait:
      self.begin()
    #self.set_state(State(Robot1_Transport))

  def wait2(self):
    logger.debug("Robot1_Wait::wait2")
    self.begin()
    self.data(Robot1_Top).robot2_wait = True
    #port = self.data(Robot1_Top)._beginOut
    #port.write()
  def begin(self):
    logger.debug("Robot1_Wait::begin")
    self.set_state(StaticFSM.State(Robot1_Transport))





@StaticFSM.fsm_substate(Robot1_Top)
class Robot1_Transport(StaticFSM.Link):
  def on_entry(self):
    logger.debug("Robot1_Transport::on_entry")
        
  def on_exit(self):
    logger.debug("Robot1_Transport::on_exit")
  def on_init(self):
    logger.debug("Robot1_Transport::on_init")
  def tock(self):
    logger.debug("Robot1_Transport::tock")
    self.movepos(self.data(Robot1_Top).robot1_hard.target, self.data(Robot1_Top).d)


  def sense(self, d):
    logger.debug("Robot1_Transport::sense")
    self.data(Robot1_Top).d = d.data
    self.sense1(0, d.data)
    
  def movepos(self, target, d):
    logger.debug("Robot1_Transport::movepos")

    
    robot1_hard = self.data(Robot1_Top).robot1_hard
    v = robot1_hard._speed*robot1_hard._sampling_time
    box = self.data(Robot1_Top).box


    guard = OpenRTM_aist.ScopedLock(robot1_hard.mutex)

    x1 = robot1_hard.x
    xb = box.x

    eps = robot1_hard._eps


    
    t = robot1_hard._K*(target - xb)
    if abs(t) > abs(v):
      robot1_hard.vel = v
    else:
      robot1_hard.vel = t
      
    #box.vel = robot1_hard.vel


      
    port = self.data(Robot1_Top)._moveOut
    data = self.data(Robot1_Top)._d_move
    OpenRTM_aist.setTimestamp(data)
    data.data = robot1_hard.vel
    port.write()
    
    

    

      
  def sense1(self, x1, xb):
    robot1_hard = self.data(Robot1_Top).robot1_hard
    eps = robot1_hard._eps
    if self.eqchk(x1,xb,eps):
      pass
    else:
      self.set_state(StaticFSM.State(Robot1_Lost))
    
    


  def begin(self):
    pass
  def lost2(self):
    self.data(Robot1_Top).robot2_wait = False
    self.set_state(StaticFSM.State(Robot1_Approach))




@StaticFSM.fsm_substate(Robot1_Top)
class Robot1_Lost(StaticFSM.Link):
  def on_entry(self):
    logger.debug("Robot1_Lost::on_entry")
        
  def on_exit(self):
    logger.debug("Robot1_Lost::on_exit")
  def on_init(self):
    logger.debug("Robot1_Lost::on_init")
    robot1_hard = self.data(Robot1_Top).robot1_hard
    robot1_hard.vel = 0
    self.set_state(StaticFSM.State(Robot1_Approach))
    port = self.data(Robot1_Top)._lost1Out
    port.write()
    self.data(Robot1_Top).robot2_wait = False

  def wait2(self):
    logger.debug("Robot1_Lost::wait2")
    self.data(Robot1_Top).robot2_wait = True

  def lost2(self):
    logger.debug("Robot1_Lost::lost2")
    self.set_state(StaticFSM.State(Robot1_Approach))