#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file Robot2.py
 @brief FSM4RTC Test
 @date $Date$

 @author 宮本　信彦　n-miyamoto@aist.go.jp
 産業技術総合研究所　ロボットイノベーション研究センター
 ロボットソフトウエアプラットフォーム研究チーム

 LGPL

"""
import sys
import time
sys.path.append(".")

# Import RTM module
import RTC
import OpenRTM_aist


import OpenRTM_aist.StaticFSM as StaticFSM
import OpenRTM_aist.EventPort as EventPort
from OpenRTM_aist.Macho import *


from Robot2Fsm import *
from robot_hard import *



# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
robot2_spec = ["implementation_id", "Robot2", 
		 "type_name",         "Robot2", 
		 "description",       "FSM4RTC Test", 
		 "version",           "1.0.0", 
		 "vendor",            "AIST", 
		 "category",          "Test", 
		 "activity_type",     "STATIC", 
		 "max_instance",      "1", 
		 "language",          "Python", 
		 "lang_type",         "SCRIPT",
		 "conf.default.start_pos", "0.3",
		 "conf.default.size", "0.1",
		 "conf.default.speed", "0.1",
		 "conf.default.K", "0.1",
		 "conf.default.EPS", "0.01",

		 "conf.__widget__.start_pos", "text",
		 "conf.__widget__.size", "text",
		 "conf.__widget__.speed", "text",
		 "conf.__widget__.K", "text",
		 "conf.__widget__.EPS", "text",

         "conf.__type__.start_pos", "double",
         "conf.__type__.size", "double",
         "conf.__type__.speed", "double",
         "conf.__type__.K", "double",
         "conf.__type__.EPS", "double",

		 ""]
# </rtc-template>

##
# @class Robot2
# @brief FSM4RTC Test
# 
# FSM4RTCのサンプル
# 
# 
class Robot2(OpenRTM_aist.DataFlowComponentBase):
	
	##
	# @brief constructor
	# @param manager Maneger Object
	# 
	def __init__(self, manager):
		OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)



		self._d_wait2 = RTC.TimedDouble(RTC.Time(0,0),0.0)
		"""
		待ち状態通知イベント送信
		 - Type: RTC::TimedLong
		"""
		self._wait2Out = OpenRTM_aist.OutPort("wait2", self._d_wait2)
		self._d_lost2 = RTC.TimedDouble(RTC.Time(0,0),0.0)
		"""
		荷物ロスト通知イベント送信
		 - Type: RTC::TimedLong
		"""
		self._lost2Out = OpenRTM_aist.OutPort("lost2", self._d_lost2)


		


		# initialize of configuration-data.
		# <rtc-template block="init_conf_param">
		"""
		初期位置
		 - Name: start_pos start_pos
		 - DefaultValue: 0.3
		 - Unit: m
		"""
		self._start_pos = [0.3]
		"""
		ロボットのサイズ
		 - Name: size size
		 - DefaultValue: 0.1
		 - Unit: m
		"""
		self._size = [0.1]
		"""
		最大速度
		 - Name: speed speed
		 - DefaultValue: 0.1
		 - Unit: m
		"""
		self._speed = [0.1]
		"""
		ゲイン定数
		 - Name: K K
		 - DefaultValue: 0.1
		"""
		self._K = [0.1]
		"""
		
		 - Name: EPS EPS
		 - DefaultValue: 0.01
		 - Unit: m
		"""
		self._EPS = [0.01]
		
		# </rtc-template>


		 
	##
	#
	# The initialize action (on CREATED->ALIVE transition)
	# formaer rtc_init_entry() 
	# 
	# @return RTC::ReturnCode_t
	# 
	#
	def onInitialize(self):
		self.robot2 = Robot_Hard(self._start_pos[0], self._speed[0], self._K[0], self._EPS[0], self._size[0], 0.01)
		self.box = Box(0, 0.1)

		"""
		イベント受信
		 - Type: RTC::TimedLong
		"""
		self._fsm = StaticFSM.Machine(Robot2_Top, self)
		self._fsm.dispatch(StaticFSM.Event(Robot2_Top.setRobot2Hard, self.robot2))
		self._fsm.dispatch(StaticFSM.Event(Robot2_Top.setBoxObj, self.box))
		self._fsm.dispatch(StaticFSM.Event(Robot2_Top.setWait2Port, self._d_wait2, self._wait2Out))
		self._fsm.dispatch(StaticFSM.Event(Robot2_Top.setLost2Port, self._d_lost2, self._lost2Out))
		
		self._eventIn = EventPort.EventInPort("event", self._fsm)
		self._eventIn.bindEvent1("sense2", Robot2_Top.sense, RTC.TimedDouble(RTC.Time(0,0),0.0),True)
		self._eventIn.bindEvent0("wait1", Robot2_Top.wait1,True)
		self._eventIn.bindEvent0("begin", Robot2_Top.begin,True)
		self._eventIn.bindEvent1("move", Robot2_Top.move1, RTC.TimedDouble(RTC.Time(0,0),0.0),True)
		self._eventIn.bindEvent0("lost1", Robot2_Top.lost1,True)
		
		# Bind variables and configuration variable
		self.bindParameter("start_pos", self._start_pos, "0.3")
		self.bindParameter("size", self._size, "0.1")
		self.bindParameter("speed", self._speed, "0.1")
		self.bindParameter("K", self._K, "0.1")
		self.bindParameter("EPS", self._EPS, "0.01")
		
		# Set InPort buffers
		self.addInPort("event",self._eventIn)
		
		# Set OutPort buffers
		self.addOutPort("wait2",self._wait2Out)
		self.addOutPort("lost2",self._lost2Out)
		
		# Set service provider to Ports
		
		# Set service consumers to Ports
		
		# Set CORBA Service Ports
		
		return RTC.RTC_OK


	def setRobot2(self, obj):
		self._fsm.dispatch(StaticFSM.Event(Robot2_Top.setRobot2Hard, obj))
		self.robot2 = obj
		
		

	def setBox(self, obj):
		self._fsm.dispatch(StaticFSM.Event(Robot2_Top.setBoxObj, obj))
		self.box = obj


	def getRobot2(self):
		return self.robot2
		

	def getBox(self):
		return self.box

	def getMachine(self):
		return self._fsm
		
		##
		# 
		# The finalize action (on ALIVE->END transition)
		# formaer rtc_exiting_entry()
		# 
		# @return RTC::ReturnCode_t
	
		# 
	def onFinalize(self):
	
		return RTC.RTC_OK
	
		##
		#
		# The startup action when ExecutionContext startup
		# former rtc_starting_entry()
		# 
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onStartup(self, ec_id):
	
		return RTC.RTC_OK
	
		##
		#
		# The shutdown action when ExecutionContext stop
		# former rtc_stopping_entry()
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onShutdown(self, ec_id):
	
		return RTC.RTC_OK
	
		##
		#
		# The activated action (Active state entry action)
		# former rtc_active_entry()
		#
		# @param ec_id target ExecutionContext Id
		# 
		# @return RTC::ReturnCode_t
		#
		#
	def onActivated(self, ec_id):
		ec = self.getExecutionContext(ec_id)
		rate = 1.0/float(ec.get_rate())
		self.robot2.init(self._start_pos[0], self._speed[0], self._K[0], self._EPS[0], self._size[0], rate)
		self._fsm.dispatch(StaticFSM.Event(Robot2_Top.StateChange))
		return RTC.RTC_OK
	
		##
		#
		# The deactivated action (Active state exit action)
		# former rtc_active_exit()
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onDeactivated(self, ec_id):
	
		return RTC.RTC_OK
	
		##
		#
		# The execution action that is invoked periodically
		# former rtc_active_do()
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onExecute(self, ec_id):
		self._fsm.dispatch(StaticFSM.Event(Robot2_Top.tock))
		
		return RTC.RTC_OK
	
		##
		#
		# The aborting action when main logic error occurred.
		# former rtc_aborting_entry()
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onAborting(self, ec_id):
	
		return RTC.RTC_OK
	
		##
		#
		# The error action in ERROR state
		# former rtc_error_do()
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onError(self, ec_id):
	
		return RTC.RTC_OK
	
		##
		#
		# The reset action that is invoked resetting
		# This is same but different the former rtc_init_entry()
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onReset(self, ec_id):
	
		return RTC.RTC_OK
	
		##
		#
		# The state update action that is invoked after onExecute() action
		# no corresponding operation exists in OpenRTm-aist-0.2.0
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#

		#
	def onStateUpdate(self, ec_id):
	
		return RTC.RTC_OK
	
		##
		#
		# The action that is invoked when execution context's rate is changed
		# no corresponding operation exists in OpenRTm-aist-0.2.0
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onRateChanged(self, ec_id):
	
		return RTC.RTC_OK
	



def Robot2Init(manager):
    profile = OpenRTM_aist.Properties(defaults_str=robot2_spec)
    manager.registerFactory(profile,
                            Robot2,
                            OpenRTM_aist.Delete)

def MyModuleInit(manager):
    Robot2Init(manager)

    # Create a component
    comp = manager.createComponent("Robot2")

def main():
	mgr = OpenRTM_aist.Manager.init(sys.argv)
	mgr.setModuleInitProc(MyModuleInit)
	mgr.activateManager()
	mgr.runManager()

if __name__ == "__main__":
	main()

