#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

import sys

import RTC
import OpenRTM_aist
import OpenRTM_aist.StaticFSM as StaticFSM
import OpenRTM_aist.EventPort as EventPort
from OpenRTM_aist.Macho import *

import random

#robot2_loss_prob = 3
#robot2_lost_length = 0.5


@StaticFSM.fsm_topstate
class Robot2_Top(StaticFSM.Link):
  def on_entry(self):
    logger.debug("Robot2_Top::on_entry")
  def on_exit(self):
    logger.debug("Robot2_Top::on_exit")
  def on_init(self):
    logger.debug("Robot2_Top::on_init")
    #self.set_state(StaticFSM.State(Robot2_Approach))
    
  def nextpos(self, d):
    pass
  def eqchk(self, x2, xb, eps):
    if(abs(x2-xb) < eps):
      return True
    else:
      return False

  def tock(self):
    pass
  def wait1(self):
    pass
  def begin(self):
    pass

  def lost1(self):
    pass

  def move1(self, v):
    pass

  def StateChange(self):
    logger.debug("Robot2_Top::StateChange")
    self.set_state(StaticFSM.State(Robot2_Approach))
    
  def setRobot2Hard(self, robot2):
    self.data(Robot2_Top).robot2_hard = robot2
  def setBoxObj(self, box):
    self.data(Robot2_Top).box = box
  def sense(self, d):
    self.data(Robot2_Top).d = d.data
  def setWait2Port(self, data, port):
    self.data(Robot2_Top)._d_wait2 = data
    self.data(Robot2_Top)._wait2Out = port
  def setLost2Port(self, data, port):
    self.data(Robot2_Top)._d_lost2 = data
    self.data(Robot2_Top)._lost2Out = port
  class Data:
    def __init__(self):
      self.robot2_hard = None
      self.box = None
      self.d = 0
      self._d_wait2 = None
      self._wait2Out = None
      self._d_lost2 = None
      self._lost2Out = None
      self.robot1_wait = False
      

@StaticFSM.fsm_substate(Robot2_Top)
class Robot2_Approach(StaticFSM.Link):
  def on_entry(self):
    logger.debug("Robot2_Approach::on_entry")
        
  def on_exit(self):
    logger.debug("Robot2_Approach::on_exit")
  def on_init(self):
    logger.debug("Robot2_Approach::on_init")

  def tock(self):
    logger.debug("Robot2_Approach::tock")
    self.nextpos(self.data(Robot2_Top).d)


  def sense(self, d):
    logger.debug("Robot2_Approach::sense")
    self.data(Robot2_Top).d = d.data
    robot2_hard = self.data(Robot2_Top).robot2_hard
    eps = robot2_hard._eps/2.0
    if self.eqchk(0,d.data,eps):
      self.set_state(StaticFSM.State(Robot2_Wait))
    
  def nextpos(self, d):
    logger.debug("Robot2_Approach::nextpos")

    robot2_hard = self.data(Robot2_Top).robot2_hard
    box = self.data(Robot2_Top).box
    #d = robot2_hard.x - (box.x + box.size)
    eps = robot2_hard._eps




    v = robot2_hard._speed*robot2_hard._sampling_time
    t = robot2_hard._K*d

    guard = OpenRTM_aist.ScopedLock(robot2_hard.mutex)

    if abs(t) > abs(v):
      robot2_hard.vel = -v
    else:
      robot2_hard.vel = -t


  def wait1(self):
    logger.debug("Robot2_Approach::wait1")
    self.data(Robot2_Top).robot1_wait = True
    
    
  def begin(self):
    pass


@StaticFSM.fsm_substate(Robot2_Top)
class Robot2_Wait(StaticFSM.Link):
  def on_entry(self):
    logger.debug("Robot2_Wait::on_entry")
        
  def on_exit(self):
    logger.debug("Robot2_Wait::on_exit")
  def on_init(self):
    logger.debug("Robot2_Wait::on_init")
    robot2_hard = self.data(Robot2_Top).robot2_hard
    robot2_hard.vel = 0
    port = self.data(Robot2_Top)._wait2Out
    port.write()
    

    if self.data(Robot2_Top).robot1_wait:
      self.begin()
    #self.set_state(State(Robot2_Transport))
  def tock(self):
    logger.debug("Robot2_Wait::tock")
    
  def wait1(self):
    logger.debug("Robot2_Wait::wait1")

    self.begin()
    #port = self.data(Robot2_Top)._wait2Out
    #port.write()
    self.data(Robot2_Top).robot1_wait = True
  def begin(self):
    logger.debug("Robot2_Wait::begin")
    self.set_state(StaticFSM.State(Robot2_Transport))



@StaticFSM.fsm_substate(Robot2_Top)
class Robot2_Transport(StaticFSM.Link):
  def on_entry(self):
    logger.debug("Robot2_Transport::on_entry")
        
  def on_exit(self):
    logger.debug("Robot2_Transport::on_exit")
  def on_init(self):
    logger.debug("Robot2_Transport::on_init")

  def wait1(self):
    pass
  def tock(self):
    logger.debug("Robot2_Transport::tock")
    #self.nextpos(self.data(Robot2).x2, self.data(Robot2).xb)
    
  def sense(self, d):
    logger.debug("Robot2_Transport::sense")
    self.data(Robot2_Top).d = d.data
    self.sense2(0, d.data)


    
    
    
    
  def move1(self, v):
    logger.debug("Robot2_Transport::move1")
    robot2_hard = self.data(Robot2_Top).robot2_hard
    box = self.data(Robot2_Top).box


    
    robot2_hard.vel = v.data
    
    
    #print robot2_hard.x - (box.x + box.size)

    
    
    


  def sense2(self, x2, xb):
    robot2_hard = self.data(Robot2_Top).robot2_hard
    eps = robot2_hard._eps
    if self.eqchk(x2,xb,eps):
      pass
    else:
      self.set_state(StaticFSM.State(Robot2_Lost))
      

  def lost1(self):
    self.data(Robot2_Top).robot1_wait = False
    self.set_state(StaticFSM.State(Robot2_Approach))
    

@StaticFSM.fsm_substate(Robot2_Top)
class Robot2_Lost(StaticFSM.Link):
  def on_entry(self):
    logger.debug("Robot2_Lost::on_entry")
        
  def on_exit(self):
    logger.debug("Robot2_Lost::on_exit")
  def on_init(self):
    logger.debug("Robot2_Lost::on_init")
    robot2_hard = self.data(Robot2_Top).robot2_hard
    robot2_hard.vel = 0
    self.data(Robot2_Top).robot1_wait = False
    self.set_state(StaticFSM.State(Robot2_Approach))
    port = self.data(Robot2_Top)._lost2Out
    port.write()
    
  def wait1(self):
    logger.debug("Robot2_Lost::wait1")
    self.data(Robot2_Top).robot1_wait = True

  def lost1(self):
    logger.debug("Robot2_Lost::lost1")
    self.set_state(StaticFSM.State(Robot2_Approach))