#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-


from PyQt4 import QtCore, QtGui
from OpenRTM_aist.Macho import *


from robot_hard import *

import sys,os

import OpenRTM_aist
from Robot1Fsm import *
from Robot2Fsm import *

import random


SamplingTime = 0.01

robot1_loss_prob = 3
robot1_lost_length = 0.5
robot2_loss_prob = 3
robot2_lost_length = 0.5
robot1_lost_eps = 0.0001
robot2_lost_eps = 0.0001

class GroundItem(QtGui.QGraphicsItem):
  def __init__(self, lx, ly, parent=None):
    super(GroundItem, self).__init__()
    self.lx = lx
    self.ly = ly
  def paint(self, painter, option, widget):
    rect = QtCore.QRect(0, 0, self.lx, self.ly)
    painter.setBrush(QtGui.QBrush(QtCore.Qt.blue, QtCore.Qt.BrushStyle(QtCore.Qt.SolidLine)))
    painter.drawRect(rect)


  def boundingRect(self):
    penWidth = 1.0
    return QtCore.QRectF(0 - penWidth / 2, 0 - penWidth / 2,
                          self.lx + penWidth, self.ly + penWidth)

class CarItem(QtGui.QGraphicsItem):
  def __init__(self, lx, ly, ground_pos, dir, parent=None):
    super(CarItem, self).__init__()
    self.lx = lx
    self.ly = ly
    self.dir = dir
    self.ground_pos = ground_pos
    self.state = "TOP"

  def paint(self, painter, option, widget):
    if self.dir == "right":
      body_rect = QtCore.QRect(0, 0, self.lx*7/8, self.ly*3/4)
      bumper_rect = QtCore.QRect(self.lx*7/8, 0, self.lx/8, self.ly*3/4)
    else:
      body_rect = QtCore.QRect(self.lx/8, 0, self.lx*7/8, self.ly*3/4)
      bumper_rect = QtCore.QRect(0, 0, self.lx/8, self.ly*3/4)

    wheel_rect = QtCore.QRect(self.lx/2-self.ly/4, self.ly/2, self.ly/2, self.ly/2)
    state_rect = QtCore.QRect(-self.lx/2, -self.ly, self.lx*2, self.ly)

    painter.setBrush(QtGui.QBrush(QtCore.Qt.green, QtCore.Qt.BrushStyle(QtCore.Qt.SolidLine)))
    painter.drawRect(body_rect)
    painter.setBrush(QtGui.QBrush(QtCore.Qt.black, QtCore.Qt.BrushStyle(QtCore.Qt.SolidLine)))
    painter.drawEllipse(wheel_rect)
    painter.setBrush(QtGui.QBrush(QtCore.Qt.red, QtCore.Qt.BrushStyle(QtCore.Qt.SolidLine)))
    painter.drawRect(bumper_rect)

    font = QtGui.QFont()
    font.setPointSize(10)
    painter.setFont(font)

    painter.drawText(state_rect, QtCore.Qt.AlignCenter, self.state)
    
        
  def boundingRect(self):
    penWidth = 1.0
    return QtCore.QRectF(-self.lx/2 - penWidth / 2, -self.ly - penWidth / 2,
                          self.lx*2 + penWidth, self.ly*2 + penWidth)


  def mousePressEvent( self, event ):
        self.cPos    = event.scenePos()
        self.pressedButton = event.button()
  
        if self.pressedButton == QtCore.Qt.RightButton:
            cursorShape = QtCore.Qt.SizeAllCursor
        else:
            cursorShape = QtCore.Qt.ClosedHandCursor
        QtGui.qApp.setOverrideCursor( QtGui.QCursor( cursorShape ) )

 
  def mouseReleaseEvent( self, event ):
        self.cPos    = None
        self.pressedButton = None
        QtGui.qApp.restoreOverrideCursor()
        super( CarItem, self ).mouseReleaseEvent( event )
        #self.setPos(10,10)
        diff = self.transform().dy()
        transform = self.transform()
        transform *= QtGui.QTransform().translate( 0, -diff )
        self.setTransform( transform )
        


  def mouseMoveEvent( self, event ):
        
        if not self.cPos:
            return
 
        # 描画位置を変更
        cur = event.scenePos()
        value = cur - self.cPos
        self.cPos = cur
        transform = self.transform()
        transform *= QtGui.QTransform().translate( value.x(), value.y() )
 
        # 変更を適用
        self.setTransform( transform )
        #print self.transform().dx(), self.transform().dy()
        #self.ground_pos-self.ly
        #self.setPos(10,10)
        #print self.pos().x()


  def setCarPos(self, x):
    transform = self.transform()
    diff = x - self.transform().dx()
    transform *= QtGui.QTransform().translate( diff, 0 )
    self.setTransform( transform )
        

class BoxItem(QtGui.QGraphicsItem):
  def __init__(self, lx, ly, ground_pos, parent=None):
    super(BoxItem, self).__init__()
    self.lx = lx
    self.ly = ly
    self.ground_pos = ground_pos

  def paint(self, painter, option, widget):

    rect = QtCore.QRect(0, 0, self.lx, self.ly)

    painter.setBrush(QtGui.QBrush(QtCore.Qt.yellow, QtCore.Qt.BrushStyle(QtCore.Qt.SolidLine)))
    painter.drawRect(rect)
  def boundingRect(self):
    penWidth = 1.0
    return QtCore.QRectF(0 - penWidth / 2, 0 - penWidth / 2,
                          self.lx + penWidth, self.ly + penWidth)
  def setBoxPos(self, x):
    transform = self.transform()
    diff = x - self.transform().dx()
    transform *= QtGui.QTransform().translate( diff, 0 )
    self.setTransform( transform )
"""
  def mousePressEvent( self, event ):
        self.cPos    = event.scenePos()
        self.pressedButton = event.button()
  
        if self.pressedButton == QtCore.Qt.RightButton:
            cursorShape = QtCore.Qt.SizeAllCursor
        else:
            cursorShape = QtCore.Qt.ClosedHandCursor
        QtGui.qApp.setOverrideCursor( QtGui.QCursor( cursorShape ) )

 
  def mouseReleaseEvent( self, event ):
        self.cPos    = None
        self.pressedButton = None
        QtGui.qApp.restoreOverrideCursor()
        super( BoxItem, self ).mouseReleaseEvent( event )
        #self.setPos(10,10)
        diff = self.transform().dy()
        transform = self.transform()
        transform *= QtGui.QTransform().translate( 0, -diff )
        self.setTransform( transform )
        


  def mouseMoveEvent( self, event ):
        
        if not self.cPos:
            return
 
        # 描画位置を変更
        cur = event.scenePos()
        value = cur - self.cPos
        self.cPos = cur
        transform = self.transform()
        transform *= QtGui.QTransform().translate( value.x(), value.y() )
 
        # 変更を適用
        self.setTransform( transform )
"""



class graphicView(QtGui.QGraphicsView):
 
    def __init__(self, scene):
      QtGui.QGraphicsView.__init__(self)
        
      scene = QtGui.QGraphicsScene(self)
      self.window_with = 800
      self.window_height = 200
      self.ground_pos = 180
      scene.setSceneRect(0, 0, self.window_with, self.window_height)
      self.setScene(scene)
      
      




      self.ground_node = GroundItem(self.window_with,self.window_height-self.ground_pos, self)
      self.ground_node.setPos(0, self.ground_pos)
      scene.addItem(self.ground_node)




      
    def transrate_gui(self, v, max_x, min_x):
      return (v+(max_x-min_x)/2)*self.window_with/(max_x-min_x)


    def size_gui(self, v, max_x, min_x):
      return v*self.window_with/(max_x-min_x)

      
    def create_box(self, x, w, max_x, min_x):
      self.box_height = 30
      self.box_width = self.size_gui(w, max_x, min_x)
      pos = self.transrate_gui(x, max_x, min_x)
      
      self.box_node = BoxItem(self.box_width, self.box_height, self.ground_pos)
      self.box_node.setPos(0, self.ground_pos-self.box_height)

      self.box_node.setBoxPos(pos)
      self.scene().addItem(self.box_node)



    def create_car1(self, x, w, max_x, min_x):
      self.car_height = 30
      self.car_width = self.size_gui(w, max_x, min_x)
      pos = self.transrate_gui(x, max_x, min_x)
      self.car1_node = CarItem(self.car_width, self.car_height, self.ground_pos, "right", self)


      self.car1_node.setPos(0, self.ground_pos-self.car_height)
      
      self.car1_node.setCarPos(pos)
      self.scene().addItem(self.car1_node)

      

    def create_car2(self, x, w, max_x, min_x):
      self.car_height = 30
      self.car_width = self.size_gui(w, max_x, min_x)
      pos = self.transrate_gui(x, max_x, min_x)

      self.car2_node = CarItem(self.car_width, self.car_height, self.ground_pos, "left", self)
      self.car2_node.setPos(0, self.ground_pos-self.car_height)
      self.car2_node.setCarPos(pos)
      self.scene().addItem(self.car2_node)


    def set_boxpos(self, x, max_x, min_x):
      pos = self.transrate_gui(x, max_x, min_x)
      self.box_node.setBoxPos(pos)

    def set_car1pos(self, x, max_x, min_x):
      pos = self.transrate_gui(x, max_x, min_x)
      self.car1_node.setCarPos(pos)

    def set_car2pos(self, x, max_x, min_x):
      pos = self.transrate_gui(x, max_x, min_x)
      self.car2_node.setCarPos(pos)

    def set_carState(self, fsm):
      if fsm.robot1_machine.is_current_direct(Robot1_Top):
        self.car1_node.state = "TOP"
      elif fsm.robot1_machine.is_current_direct(Robot1_Approach):
        self.car1_node.state = "Approach"
      elif fsm.robot1_machine.is_current_direct(Robot1_Wait):
        self.car1_node.state = "Wait"
      elif fsm.robot1_machine.is_current_direct(Robot1_Transport):
        self.car1_node.state = "Transport"
      elif fsm.robot1_machine.is_current_direct(Robot1_Lost):
        self.car1_node.state = "Lost"
      
      if fsm.robot2_machine.is_current_direct(Robot2_Top):
        self.car2_node.state = "TOP"
      elif fsm.robot2_machine.is_current_direct(Robot2_Approach):
        self.car2_node.state = "Approach"
      elif fsm.robot2_machine.is_current_direct(Robot2_Wait):
        self.car2_node.state = "Wait"
      elif fsm.robot2_machine.is_current_direct(Robot2_Transport):
        self.car2_node.state = "Transport"
      elif fsm.robot2_machine.is_current_direct(Robot2_Lost):
        self.car2_node.state = "Lost"
      
      
      


class FSM_Machine:
  def __init__(self, robot1, robot2, box, robot1_machine, robot2_machine):
    self.robot1_machine = robot1_machine
    self.robot2_machine = robot2_machine
    
    self.robot1 = robot1
    self.robot2 = robot2
    self.box = box
    


class WindowWidget(QtGui.QWidget):
  def __init__(self, robot1, robot2, box, fsm=None, parent=None):
    super(WindowWidget, self).__init__(parent)
    mainLayout = QtGui.QVBoxLayout()
    
    self.setLayout(mainLayout)



    self.scene = QtGui.QGraphicsScene(0, 0, 600, 600)
  

    self.view = graphicView(self.scene)

    mainLayout.addWidget(self.view)
    

    self.robot1 = robot1
    self.robot2 = robot2
    self.box = box
    
    self.fsm = fsm

    self.max_x = 1.0
    self.min_x = -1.0

    
    
    
    self.view.create_box(self.box.x, self.box.size, self.max_x, self.min_x)
    self.view.create_car1(self.robot1.x, self.robot1._size, self.max_x, self.min_x)
    self.view.create_car2(self.robot2.x, self.robot2._size, self.max_x, self.min_x)


    if self.fsm:
      self.timer = QtCore.QTimer()
      self.timer.setInterval(SamplingTime*1000)
      self.timer.timeout.connect(self.update_machine)
      self.timer.start()


    subLayout = QtGui.QHBoxLayout()
    mainLayout.addLayout(subLayout)

    robot1layout = QtGui.QVBoxLayout()
    robot1layout.addWidget(QtGui.QLabel(u'ロボット1'))
    
    
    self.robot1_loss_prob_textbox = QtGui.QLineEdit()
    self.robot1_loss_prob_textbox.setText(str(robot1_loss_prob))

    subsublayout = QtGui.QVBoxLayout()
    subLayout.addLayout(robot1layout)
    robot1layout.addLayout(subsublayout)
    subsublayout.addWidget(QtGui.QLabel(u'ロストする確率'))
    subsublayout.addWidget(self.robot1_loss_prob_textbox)


    
    self.robot1_lost_length_textbox = QtGui.QLineEdit()
    self.robot1_lost_length_textbox.setText(str(robot1_lost_length))
    

    subsublayout = QtGui.QVBoxLayout()
    robot1layout.addLayout(subsublayout)
    subsublayout.addWidget(QtGui.QLabel(u'離れる距離'))
    subsublayout.addWidget(self.robot1_lost_length_textbox)

    
    self.robot1_lost_eps_textbox = QtGui.QLineEdit()
    self.robot1_lost_eps_textbox.setText(str(robot1_lost_eps))
    
  
    subsublayout = QtGui.QVBoxLayout()
    robot1layout.addLayout(subsublayout)
    subsublayout.addWidget(QtGui.QLabel(u'ロストが発生する最低速度'))
    subsublayout.addWidget(self.robot1_lost_eps_textbox)



    robot2layout = QtGui.QVBoxLayout()
    robot2layout.addWidget(QtGui.QLabel(u'ロボット2'))

    self.robot2_loss_prob_textbox = QtGui.QLineEdit()
    self.robot2_loss_prob_textbox.setText(str(robot2_loss_prob))


    subsublayout = QtGui.QVBoxLayout()
    subLayout.addLayout(robot2layout)
    robot2layout.addLayout(subsublayout)
    subsublayout.addWidget(QtGui.QLabel(u'ロストする確率'))
    subsublayout.addWidget(self.robot2_loss_prob_textbox)


    self.robot2_lost_length_textbox = QtGui.QLineEdit()
    self.robot2_lost_length_textbox.setText(str(robot2_lost_length))




    subsublayout = QtGui.QVBoxLayout()
 
    robot2layout.addLayout(subsublayout)
    subsublayout.addWidget(QtGui.QLabel(u'離れる距離'))
    subsublayout.addWidget(self.robot2_lost_length_textbox)



    self.robot2_lost_eps_textbox = QtGui.QLineEdit()
    self.robot2_lost_eps_textbox.setText(str(robot2_lost_eps))



    subsublayout = QtGui.QVBoxLayout()

    robot2layout.addLayout(subsublayout)
    subsublayout.addWidget(QtGui.QLabel(u'ロストが発生する最低速度'))
    subsublayout.addWidget(self.robot2_lost_eps_textbox)
    

  def update_simulation(self):
    global robot1_loss_prob
    global robot2_loss_prob
    global robot1_lost_length
    global robot2_lost_length
    global robot1_lost_eps
    global robot2_lost_eps
    
    r1_lp = robot1_loss_prob
    r2_lp = robot2_loss_prob
    r1_ll = robot1_lost_length
    r2_ll = robot2_lost_length
    r1_eps = robot1_lost_eps
    r2_eps = robot2_lost_eps
    
    try:
      r1_lp = int(self.robot1_loss_prob_textbox.text())
      robot1_loss_prob = r1_lp
    except:
      pass
    try:
      r2_lp = int(self.robot2_loss_prob_textbox.text())
      robot2_loss_prob = r2_lp
    except:
      pass
    try:
      r1_ll = float(self.robot1_lost_length_textbox.text())
      robot1_lost_length =  r1_ll
    except:
      pass
    try:
      r2_ll = float(self.robot2_lost_length_textbox.text())
      robot2_lost_length = r2_ll
    except:
      pass
    try:
      r1_eps = float(self.robot1_lost_eps_textbox.text())
      robot1_lost_eps = r1_eps
    except:
      pass
    try:
      r2_eps = float(self.robot2_lost_eps_textbox.text())
      robot2_lost_eps = r2_eps
    except:
      pass
    
    
      
    guard = OpenRTM_aist.ScopedLock(self.robot1.mutex)
    #self.box.x = self.box.next_x
    
    
    if abs(self.robot1.vel) > r1_eps:
      if random.randint(1,10000) > r1_lp:
        pass
      else:
        self.robot1.x -= r1_ll*random.random()
    self.robot1.x += self.robot1.vel
    
    if abs(self.robot2.vel) > r2_eps:
      if random.randint(1,10000) > r2_lp:
        pass
      else:
        self.robot2.x += r2_ll*random.random()
    self.robot2.x += self.robot2.vel

    if self.box.x < self.robot1.x:
      self.box.x = self.robot1.x

    elif self.box.x+self.box.size > self.robot2.x:
      self.box.x = self.robot2.x - self.box.size
      if self.box.x < self.robot1.x:
        self.box.x = self.robot1.x
      

  def update_machine(self):
    if self.fsm:
      self.update_simulation()
      self.view.set_boxpos(self.box.x,self.max_x, self.min_x)
      self.view.set_car1pos(self.robot1.x-self.box.size,self.max_x, self.min_x)
      self.view.set_car2pos(self.robot2.x,self.max_x, self.min_x)
      self.view.set_carState(self.fsm)
      self.view.car1_node.update()
      self.view.car2_node.update()

class MainWindow(QtGui.QMainWindow):
  def __init__(self, robot1, robot2, box, fsm=None):
    super(MainWindow, self).__init__()
    self.setWindowTitle(u"協調ロボット監視GUI")
    self.window = WindowWidget(robot1, robot2, box, fsm, self)
    self.setCentralWidget(self.window)
    



def main():
  manager = OpenRTM_aist.Manager.init(sys.argv)

  manager.activateManager()

  manager.runManager(True)
  robot1_comp = manager.getComponent("Robot10")
  robot2_comp = manager.getComponent("Robot20")
  sensor1_comp = manager.getComponent("Sensor10")
  sensor2_comp = manager.getComponent("Sensor20")
  robot1 = robot1_comp.getRobot1()
  box = robot1_comp.getBox()
  robot2 = robot2_comp.getRobot2()
  robot2_comp.setBox(box)
  sensor1_comp.setRobot1(robot1)
  sensor1_comp.setBox(box)
  sensor2_comp.setRobot2(robot2)
  sensor2_comp.setBox(box)

  robot1_machine = robot1_comp.getMachine()
  robot2_machine = robot2_comp.getMachine()

  ec = sensor1_comp.getExecutionContext(0)
  ec.activate_component(sensor1_comp.getObjRef())
  ec = sensor2_comp.getExecutionContext(0)
  ec.activate_component(sensor2_comp.getObjRef())
  ec = robot1_comp.getExecutionContext(0)
  robot1._sampling_time = 1.0/float(ec.get_rate())
  ec.activate_component(robot1_comp.getObjRef())
  ec = robot2_comp.getExecutionContext(0)
  ec.activate_component(robot2_comp.getObjRef())
  robot2._sampling_time = 1.0/float(ec.get_rate())


  mutex = robot1.mutex
  robot2.mutex = mutex
  box.mutex = mutex
  
  app = QtGui.QApplication([""])
  fsm = FSM_Machine(robot1, robot2, box, robot1_machine, robot2_machine)
  mainWin = MainWindow(robot1, robot2, box, fsm)
  mainWin.show()
  app.exec_()


############### test #################
if __name__ == '__main__':
   main()
